# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .agents_search_deployed_agents_request_search_item_direction_value import (
    AgentsSearchDeployedAgentsRequestSearchItemDirectionValue,
)
from .agents_search_deployed_agents_request_search_item_direction_direction import (
    AgentsSearchDeployedAgentsRequestSearchItemDirectionDirection,
)
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class AgentsSearchDeployedAgentsRequestSearchItemDirection(UniversalBaseModel):
    field: typing.Literal["order_by"] = "order_by"
    value: AgentsSearchDeployedAgentsRequestSearchItemDirectionValue
    direction: AgentsSearchDeployedAgentsRequestSearchItemDirectionDirection

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
