# This file was auto-generated by Fern from our API Definition.

import typing

AgentsSearchDeployedAgentsRequestSearchItemDirectionValue = typing.Union[
    typing.Literal["created_at", "updated_at"], typing.Any
]
