# This file was auto-generated by Fern from our API Definition.

import typing

AgentType = typing.Union[
    typing.Literal["memgpt_agent", "split_thread_agent", "o1_agent", "offline_memory_agent", "chat_only_agent"],
    typing.Any,
]
