# This file was auto-generated by Fern from our API Definition.

import typing

AgentsSearchDeployedAgentsRequestSearchItemOperatorOperator = typing.Union[
    typing.Literal["eq", "neq", "contains"], typing.Any
]
