# This file was auto-generated by Fern from our API Definition.

import typing

AgentsSearchDeployedAgentsRequestSearchItemDirectionDirection = typing.Union[typing.Literal["asc", "desc"], typing.Any]
