# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .letta_schemas_openai_chat_completion_response_tool_call import LettaSchemasOpenaiChatCompletionResponseToolCall
from .function_call_output import FunctionCallOutput
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class LettaSchemasOpenaiChatCompletionResponseMessage(UniversalBaseModel):
    content: typing.Optional[str] = None
    tool_calls: typing.Optional[typing.List[LettaSchemasOpenaiChatCompletionResponseToolCall]] = None
    role: str
    function_call: typing.Optional[FunctionCallOutput] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
